// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// MODULE:   Eject.c
//
// PURPOSE:  This file contains the code that ejects a removable device's media.
//
// COMMENTS
//
// This file was last modified on 1/21/99 by Max Feingold (maf6@cornell.edu)
//
// Max is not responsible for the misuse of this file or its corresponding
// executable.  As far as Max is concerned you can do whatever you want with either.

#include "Eject.h"
#include <winioctl.h>
#include <stdio.h>

BOOL EjectVolume (TCHAR* pszDriveName, TCHAR* pszReturnMessage) {

	TCHAR pszTemp [1024];
	DWORD dwLastError, dwDriveAttributes, dwBytesReturned;
	HANDLE hRemovable;

	PREVENT_MEDIA_REMOVAL pmrMediaRemoval;

	// Check volume
	if (_tcslen (pszDriveName) != 2 || !_istalpha (pszDriveName[0]) || pszDriveName[1] != ':') {
		_stprintf (pszReturnMessage, _T("Usage:  Eject [computer name] <driveletter>:\n"));
		return FALSE;
	}

	// Check for drive existence, media present in drive
	_tcscpy (pszTemp, pszDriveName);
	_tcscat (pszTemp, _T("\\."));

	SetErrorMode (SEM_FAILCRITICALERRORS);
    dwDriveAttributes = GetFileAttributes (pszDriveName);  
    SetErrorMode (0); 

	if (dwDriveAttributes == 0xffffffff) {

		dwLastError = GetLastError();

		if (dwLastError == ERROR_INVALID_HANDLE) {
			_stprintf (pszReturnMessage, _T("The device does not exist.\n"));
		}

		else if (dwLastError == ERROR_NOT_READY) {
			_stprintf (pszReturnMessage, _T("The device does not contain storage media.\n"));
		}

		else _stprintf (pszReturnMessage, _T("The device is not responding.\n"));

		return FALSE;
	}

	// Make sure drive is removable
	dwDriveAttributes = GetDriveType (pszDriveName);
	if (dwDriveAttributes != DRIVE_REMOVABLE && dwDriveAttributes != DRIVE_CDROM) {
		_stprintf (pszReturnMessage, _T("The device is not removable.\n"));
		return FALSE;
	}

	// Get a handle to the volume
	_tcscpy (pszTemp, _T("\\\\.\\"));
	_tcscat (pszTemp, pszDriveName);

	hRemovable = CreateFile (
		pszTemp,
		dwDriveAttributes == DRIVE_CDROM ? GENERIC_READ : GENERIC_READ | GENERIC_WRITE,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL,
		OPEN_EXISTING,
		0,
		NULL);
	
	if (hRemovable == INVALID_HANDLE_VALUE) {
		_stprintf (pszReturnMessage, _T("You do not have permission to access the device.\n"));
		return FALSE;
	}
	
	// Flush the dirty buffers of the volume
	FlushFileBuffers (hRemovable);

	// Make sure the device can be ejected
	pmrMediaRemoval.PreventMediaRemoval = FALSE;
	if (!DeviceIoControl (
		hRemovable,
		IOCTL_STORAGE_MEDIA_REMOVAL,
		&pmrMediaRemoval,
		sizeof (PREVENT_MEDIA_REMOVAL),
		NULL,
		0,
		&dwBytesReturned,
		NULL)) {

		_stprintf (pszReturnMessage, _T("The device could not be ejected.\n"));
		
		CloseHandle (hRemovable);

		return FALSE;
	}

	// Eject the device
	if (!DeviceIoControl (
		hRemovable,
		IOCTL_STORAGE_EJECT_MEDIA,
		NULL,
		0,
		NULL,
		0,
		&dwBytesReturned,
		NULL)) {

		_stprintf (pszReturnMessage, _T("The device could not be ejected.\n"));
		
		CloseHandle (hRemovable);

		return FALSE;
	}

	_stprintf (pszReturnMessage, _T("The device was ejected successfully.\n"));

	CloseHandle (hRemovable);

	return TRUE;
}